#
#
# Fonctions interactives permettant de faire des choix
#  a l'ecran, en utilisant la souris et des fenetres
#  graphiques qui le font bien !
#
# Creation: 2005/2/16 - A. Sarrat
# Modifie: 2005/2/19 -  F. Schuller
#          .... 2005/10/28
# Last modif: 2005/11/04 - adapted to BoGLi
#

from ppgplot import *
import string


class Fenetre:

    """
    classe Fenetre - parametres et methodes pour les boites et boutons
    attributs:
    int forme    : 0=cercle 1=rectangle 2=rectangle transparent
    list pos     : positions (X,Y) des centres dans la fenetre
    list float/tuple size: rayon ou (largeur,hauteur)
    list label   : messages a apparaitre (vecteur de string) dans ou pres d'une fenetre
    tuple txtpos : position des labels relativement a pos
    int font     : taille des caracteres
    int coltxt   : couleur du texte
    int colfond  : couleur de fond des boutons
    int family   : police de caracteres
    """

    def __init__(
        self, forme=0, pos=[], size=0, label=[], txtpos=(0, 0), font=2.,
            coltxt=1, colfond=1, family=1):
        self.forme = forme
        self.pos = pos
        self.size = size
        self.label = label
        self.txtpos = txtpos
        self.font = font
        self.coltxt = coltxt
        self.colfond = colfond
        self.family = family

    def dessine(self, new=0):
        """
        method dessine

        INP: new : efface la fenetre si non nul
        OUT: aucune
        """
        if new:
            pgeras()
        pgsfs(2)
        # pgrect(0,1000,500,0)
        pgsave()                # sauvegarde les attributs de la figure
        pgbbuf()                # demarre la preparation de la page (batch)
        pgslw(2)                # definit l'epaisseur des lignes
        pgsfs(1)                # definit le style de remplissage
        pgsch(self.font)        # definit la taille des caracteres
        pgscf(self.family)      # definit la fonte

        # Cree une mini-page temporaire de selection
        for i in range(len(self.label)):
            pgsci(self.colfond)
                  # definit l'index de couleur 1:blanc 2:rouge...
            x0 = self.pos[i][0]
            y0 = self.pos[i][1]
            if self.forme == 1:
                oneSize = self.size[i]
                pgrect(x0 - oneSize[0] / 2., x0 + oneSize[0] / 2.,
                       y0 - oneSize[1] / 2., y0 + oneSize[1] / 2.,)
            elif self.forme == 0:
                oneSize = (self.size[i], self.size[i])
                pgcirc(x0, y0, self.size[i])
            pgsci(self.coltxt)
            if '|' in self.label[i]:
                # allow for two words, separated on two lines
                words = string.split(self.label[i], '|')
                pgptxt(
                    x0 + self.txtpos[0], y0 + self.txtpos[1] - oneSize[1] / 4.,
                    0.0, 0.5, words[0])
                pgptxt(
                    x0 + self.txtpos[0], y0 + self.txtpos[1] + oneSize[1] / 4.,
                    0.0, 0.5, words[1])
            else:
                pgptxt(
                    x0 + self.txtpos[0],
                    y0 + self.txtpos[1],
                    0.0,
                    0.5,
                    self.label[i])
        pgebuf()       # fin de la preparation de la page
        pgunsa()       # retourne aux attributs de pgplot

    def saisie(self):
        """
        method saisie

        INP: aucune
        OUT: choix : selection (numero du bouton)
        """

        # Selection d'un bouton de reponse par l'utilisateur (a la souris)
        again = 1
        xx, yy = 500, 150
        while(again):
            xx, yy, zz = pgcurs(xx, yy)
            choix = -1
            for i in range(len(self.label)):
                x0 = self.pos[i][0]
                y0 = self.pos[i][1]
                if self.forme:
                    oneSize = self.size[i]
                    if (abs(xx - x0) <= oneSize[0] / 2. and
                       abs(yy - y0) <= oneSize[1] / 2.):
                        choix = i
                        again = 0
                else:
                    oneSize = (self.size[i], self.size[i])
                    if ((xx - x0) ** 2 + (yy - y0) ** 2) <= oneSize ** 2:
                        choix = i
                        again = 0

        return choix


def fenetreInteractive(
    x0=600.,
    x1=675.,
    y0=400.,
    y1=480,
    prevText='',
        bgCol=0):
    """
    method fenetreInteractive():
    INP: (float) x0,x1,y0,y1: box coordinates
         (str) prevText: previous field value
         (int) bgCol: background color

    OUT: (str) value: user input
    """

    box = [x0, x1, y0, y1]
    X = x0 + 10.
    Y = y1 - 10.

    pgsave()                # sauvegarde les attributs de la figure
    pgbbuf()                # demarre la preparation de la page (batch)
    pgsci(16)               # definit l'index de couleur 1:blanc 2:rouge...
    pgslw(2)                # definit l'epaisseur des lignes
    pgsfs(1)                # definit le style de remplissage
    pgsch(2.0)              # definit la taille des caracteres
    pgscf(1)                # definit la fonte

    # black frame
    pgrect(x0, x1, y0, y1)

    # input text in red
    pgsci(2)
    value = pgrstr(X, Y, 0.0, 0.0, prevText, len(prevText), bgCol)
    value = value.rstrip()

    pgebuf()       # fin de la preparation de la page
    pgunsa()       # retourne aux attributs de pgplot

    return value


def pgrstr(X, Y, ANGLE, FJUST, TEXT, LSTR, BCI):
    """
    method pgrstr(X, Y, ANGLE, FJUST, TEXT, LSTR, BCI)
    Lit une chaine de caractere dans la fenetre courante
    INP: X, Y, ANGLE, FJUST = position angle et justification du texte
         BCI = couleur de la fenetre
    INP/OUT: TEXT, LSTR = texte et longueur de la chaine TEXT
    """

    CI = pgqci()
    again = 1
    while(again):
        # Draw current string
        if (LSTR > 0):
            pgptxt(X, Y, ANGLE, FJUST, TEXT[0:LSTR])
            JUNK = pgqtxt(X, Y, ANGLE, FJUST, TEXT[0:LSTR])
            XCUR = JUNK[2][0]
            YCUR = JUNK[2][1]
        else:
            XCUR = X
            YCUR = Y

        # Read a character
        XX, YY, CH = pgband(0, 1, XCUR, YCUR)

        # Erase old string
        pgsci(BCI)
        if (LSTR > 0):
            pgptxt(X, Y, ANGLE, FJUST, TEXT[0:LSTR])
        pgsci(CI)

        # Avoid problem with PGPLOT escape character
        if (CH == chr(92)):
            CH = '*'

        # Backspace (ctrl H) or delete remove entire string
        if (ord(CH) == 8 or ord(CH) == 127):
            if (LSTR > 0):
                LSTR = 0
                TEXT = ""

        # Any other non-printing character terminates input
        elif (ord(CH) < 32):
            if (LSTR > 0):
                pgptxt(X, Y, ANGLE, FJUST, TEXT[0:LSTR])
            again = 0

        # Otherwise, add character to string if there is room
        else:
            LSTR = LSTR + 1
            TEXT = TEXT + CH

    return TEXT

# @}



# Copyright (C) 2014
# Institut d'Astrophysique Spatiale
#
# Forked from the BoA project
#
# Copyright (C) 2002-2006
# Max-Planck-Institut fuer Radioastronomie Bonn
# Argelander Institut fuer Astronomie
# Astronomisches Institut der Ruhr-Universitaet Bochum
#
# Produced for the LABOCA project
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Library General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library General Public License for more
# details.
#
# You should have received a copy of the GNU Library General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 675 Massachusetts Ave, Cambridge, MA 02139, USA.
#
#
# update started 08/12/06 by C.Vlahakis
# update continued 09/03/07 by C. Vlahakis
# more updating and re-organisation on 20.03.07 by C. Vlahakis
#

"""
DES: Little module to define Rea Shortcut, updated
USE: execfile('ReaShortcut.py') to reset the current shortcut
"""

# from rea import ReaReduce
from rea import ReaConfig, ReaDir
from rea import ReaMessageHandler
from rea import ReaMapping, ReaPointing, ReaFocus
from rea.Bogli import *

# Baseline, noise and statistics related
#
polynomialBaseline = baseline = base = data.polynomialBaseline
medianNoiseRemoval = mediannoise = data.medianNoiseRemoval
statistics = stats = stat = data._DataAna__statistics
computeRms = maprms = data.computeRmsFromMap
medianBaseline = medianBase = medianbase = data.medianBaseline


# Mapping related
#
mapping = doMap = domap = data.doMap
smoothBy = smooth = data.smoothMap
display = mapdisp = mapdisplay = data.displayMap
chanMap = ChanMap = chanmap = data.chanMap
computeWeight = computeweight = weight = data.computeWeight
showMap = data.showMap
mapsum = mapSum = ReaMapping.mapSum
mapsum2 = mapSum2 = ReaMapping.mapSum2
mapsumfast = mapSumFast = ReaMapping.mapsumfast
zoom = data.zoom
setValPoly = setValuesPolygon = ReaMapping.setValuesPolygon

# Plotting related
#
plotArray = plotarray = data.ReceiverArray.plotArray
plotAzimuth = azimuth = azim = az = data.ScanParam.plotAzimuth
plotElevation = elevation = elev = el = data.ScanParam.plotElevation
plotAzEl = azel = data.ScanParam.plotAzEl
plotAzimuthOffset = azimuthOffset = azimoff = azo = data.ScanParam.plotAzimuthOffset
plotElevationOffset = elevationOffset = eleoff = elo = data.ScanParam.plotElevationOffset
plotAzElOffset = azeloff = azelo = data.ScanParam.plotAzElOffset
plotAzElSpeed = azelspeed = azelsp = data.ScanParam.plotAzElSpeed
plotAzElAcceleration = azelaccel = azelac = data.ScanParam.plotAzElAcceleration
plotCorrel = plotcorrel = plotcor = plotCor = data.plotCorrel
plotFFT = data.plotFFT
plotMean = plotmean = data.plotMean
plotRms = plotrms = data.plotRms
plotMeanChan = plotmeanchan = data.plotMeanChan
plotRmsChan = plotrmschan = data.plotRmsChan
plotSubscan = plotSub = data.ScanParam.plotSubscan
plotSubscanOffsets = plotSubOff = data.ScanParam.plotSubscanOffsets
signal = signa = sign = sig = data.signal


# Pointing and Focus related
#
solvePointingOnMap = solvepointing = solvepoint = solvepoin = solvepoi = data.solvePointingOnMap
solveFocus = solvefocus = solveFoc = solvefoc = data.solveFocus


# Data handling related
#
setCurrChanList = channels = channel = chan = data.ReceiverArray.setCurrChanList
checkChanList = checkChannels = checkChan = data.ReceiverArray.checkChanList
printCurrChanList = printChannels = printChan = data.ReceiverArray.printCurrChanList
getPixel = getPix = data.getPixelFromMap


# Bogli Related
#
selectDev = device = dev = DeviceHandler.selectDev
clear = cle = cl = Plot.clear
openDev = op = DeviceHandler.openDev
closeDev = close = clo = cls = DeviceHandler.closeDev
pointSize = BogliConfig.pointSize
poly = defPoly = defPolygon = Plot.defPolygon
savePoly = savePolygon = Plot.savePolygon
loadPoly = loadPolygon = Plot.loadPolygon
plotPoly = plotPolygon = Plot.plotPolygon

# SNF related
#
correlatedNoiseRemoval = cnr = CNR = data.correlatedNoiseRemoval
pca = PCA = corrpca = corrPCA = data.corrPCA


# Save & restore
#
dumpData = dump = data.dumpData
restoreData = restore = data.restoreData


# Flag related
#
flag = data.flag
flagMJD = data.flagMJD
flagLon = data.flagLon
flagRms = data.flagRms
flagSpeed = flagS = data.flagSpeed
flagAccel = flagA = data.flagAccel
flagFractionRms = flagFRms = data.flagFractionRms
flagPosition = flagPos = data.flagPosition
flagSubscan = flagSub = data.flagSubscan  # do we want this???
flagChannels = flagC = flagCh = fCh = data.flagChannels
unflag = data.unflag
unflagChannels = unflagC = unflagCh = ufCh = data.unflagChannels
despike = dspike = data.despike
itDespike = iterativeDespike = data.iterativeDespike


# I/O related
#
findInDir = find = fd = ReaDir.findInDir
removeScans = remove = rs = ReaDir.removeScans
selectInDir = select = slt = ReaDir.selectInDir
listInDir = ils = inls = ReaDir.listInDir
setInDir = indir = ind = ReaDir.setInDir
setInFile = infile = inf = ReaDir.setInFile
setOutFile = outfile = outf = ReaDir.setOutFile
setOutDir = outdir = outd = ReaDir.setOutDir
setProjectID = setproj = proj = ReaDir.setProjectID
setDate = setdate = date = ReaDir.setDate
resetCurrentList = resetCurrList = rls = ReaDir.resetCurrentList
read = data.read


# Misc
#
rcp = readRCP = readRCPfile = data.ReceiverArray.readRCPfile
restore = restoreData = data.restoreData
updateRCP = data.ReceiverArray.updateRCP
flagRCP = data.flagRCP
flatfield = flat = data.flatfield
correctOpacity = opacity = opac = data.correctOpacity
flattenFreq = data.flattenFreq
blankFreq = data.blankFreq
addSource = addSourceModel = data.addSource  # defined in ReaMapping
# saveMambo = mambo = data.saveMambo


# MessageHandler related
#
# mess=messages.setMess
# messweight=messweigh=messweig=messwei=messwe=messw=ReaB.MessHand.setMaxWeight
messweight = messw = data.MessHand.setMaxWeight



from numpy.distutils.core import setup, Extension

setup(name="rea",
      version="0.1.1",
      author="Alexandre Beelen",
      author_email="alexandre.beelen@ias.u-psud.fr",

      url="http://www.ias.u-psud.fr/abeelen/rea",
      download_url = "http://www.ias.u-psud.fr/abeelen/rea/download",
      description="REceiver Array Analysis software",

      classifiers = [
          "Programming Language :: Python",
          "Development Status :: 4 - Beta",
          "Environment :: X11 Applications",
          "Intended Audience :: Science/Research",
          "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
          "Natural Language :: English",
          "Programming Language :: Fortran",
          "Topic :: Scientific/Engineering :: Astronomy",
          ],
      packages=["rea", "rea.Bogli", "rea.fortran"],
      package_data={'rea.Bogli': ['lut/*.lut']},
      scripts=['bin/rea'],

      package_dir={'rea': '.',
                   'rea.Bogli': 'Bogli',
                   'rea.fortran': 'fortran'},
      ext_modules=[
          Extension(
              'rea.fortran.fBaseline',
              ['fortran/fBaseline.f90', 'fortran/fFit.f90'],
              extra_link_args=['-llapack -lblas']),
          Extension(
              'rea.fortran.fFit',
              ['fortran/fFit.f90'],
              extra_link_args=['-llapack -lblas']),
          Extension(
              'rea.fortran.fFlagHandler',
              ['fortran/fFlagHandler.f90']),
          Extension(
              'rea.fortran.fFlag',
              ['fortran/fFlag.f90']),
          Extension(
              'rea.fortran.fMap',
              ['fortran/fMap.f90'],
              extra_link_args=['-llapack -lblas']),
          Extension(
              'rea.fortran.fSNF',
              ['fortran/fSNF.f90', 'fortran/fFit.f90'],
              extra_link_args=['-llapack -lblas']),
          Extension(
              'rea.fortran.fStat',
              ['fortran/fStat.f90']),
          Extension(
              'rea.fortran.fUtilities',
              ['fortran/fUtilities.f90']),
          Extension(
              'rea.fortran.fWavelets',
              ['fortran/fWavelets.f90'])
      ],



)



# Copyright (C) 2002-2006
# Max-Planck-Institut fuer Radioastronomie Bonn
# Argelander Institut fuer Astronomie
# Astronomisches Institut der Ruhr-Universitaet Bochum
#
# Produced for the LABOCA project
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Library General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library General Public License for more
# details.
#
# You should have received a copy of the GNU Library General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 675 Massachusetts Ave, Cambridge, MA 02139, USA.
#

"""
DES: module used to plot different symbols and forms
"""

__version__ = '$Revision: 2314 $'
__data__ = '$Date: 2007-08-08 10:43:50 +0200 (mer, 08 aou 2007) $'

from . import Plot
from ppgplot import *
import numpy as np


def ellipse(x, y, xFWHM, yFWHM, tilt, overplot=1, style='l', ci=1):
    """
    DES: draw a ellipse
    INP: (f) x/y      : the position offset
         (f) x/y FWHM : the minor and major axis
         (f) tilt     : the angle of the ellipse (radian)
         (l) overplot : should we overplot the ellipse (default: yes)
         (s) style    : the style of the plot (default: line)
         (i) ci       : the color index (default white)
    """

    theta = np.arange(31.) / 30 * 2 * np.pi

    xp = xFWHM / 2. * np.cos(theta)
    yp = yFWHM / 2. * np.sin(theta)

    ix = xp * np.cos(-1. * tilt) - yp * np.sin(-1. * tilt)
    iy = xp * np.sin(-1. * tilt) + yp * np.cos(-1. * tilt)

    ix = ix + x
    iy = iy + y

    Plot.plot(ix, iy, overplot=overplot, style=style, ci=ci)


def circle(x, y, radius, fill=0, ci=1):
    """
    DES: draw a (filled) circle using pgplot primitive
    INP: (f) x/y      : the position offset
         (f) radius   : the radisu of the circle
         (s) fill     : should we fill the circle  (default: no)
         (i) ci       : the color index (default white)
    """
    savedFillStyle = pgqfs()

    if fill:
        pgsfs(1)
    else:
        pgsfs(2)

    pgsci(ci)
    pgcirc(x, y, radius)

    pgsfs(savedFillStyle)


def shadeY(Ymin, Ymax, style=3, ci=1):
    """
    DES: shape an area between two Y values
    INP: (f) Y min/max : the limits (could be off plot limits)
         (i) style     : the shading style (see pgplot help)
         (i)    ci     : the color of the shade
    """
    savedFillStyle = pgqfs()

    pgsfs(style)
    pgsci(ci)
    xAxis = Plot.xAxis['limits']
    pgpoly(np.array([xAxis[0], xAxis[0], xAxis[1], xAxis[1]]),
           np.array([Ymin, Ymax, Ymax, Ymin]))
    pgsfs(savedFillStyle)


def shadeX(Xmin, Xmax, style=3, ci=1):
    """
    DES: shape an area between two X values
    INP: (f) X min/max : the limits (could be off plot limits)
         (i) style     : the shading style (see pgplot help)
         (i)    ci     : the color of the shade
    """
    savedFillStyle = pgqfs()

    pgsfs(style)
    pgsci(ci)
    yAxis = Plot.yAxis['limits']
    pgpoly(np.array([Xmin, Xmax, Xmax, Xmin]),
           np.array([yAxis[0], yAxis[0], yAxis[1], yAxis[1]]))
    pgsfs(savedFillStyle)
